package dto.extension;

import java.io.IOException;
import java.io.InputStream;
import java.util.logging.Logger;

import javax.enterprise.event.Observes;

import javax.enterprise.inject.spi.*;
import javax.xml.parsers.*;

import dto.*;
import org.w3c.dom.*;

import org.xml.sax.SAXException;


public class BaseDtoExtension implements Extension {
	private final Document document;
	private final Logger log = Logger.getLogger(BaseDtoExtension.class
			.getName());

	public BaseDtoExtension() {
		try {
			InputStream creatureDefs = BaseDtoExtension.class.getClassLoader()
					.getResourceAsStream("inject-beans.xml");
			DocumentBuilderFactory factory = DocumentBuilderFactory
					.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
			document = builder.parse(creatureDefs);
		} catch (ParserConfigurationException e) {
			throw new RuntimeException("Error building xml parser, aborting", e);
		} catch (SAXException e) {
			throw new RuntimeException("SAX exception while parsing xml file",
					e);
		} catch (IOException e) {
			throw new RuntimeException("Error reading or parsing xml file", e);
		}
	}


	<X extends BaseDto> void processInjectionTarget(
			@Observes ProcessInjectionTarget<X> pit) {
		Class<? extends BaseDto> klass = pit.getAnnotatedType().getJavaClass();
		log.info("Setting up injection target for " + klass);
		final Element entry = (Element) document.getElementsByTagName(
				klass.getSimpleName().toLowerCase()).item(0);
		pit.setInjectionTarget(new XmlWrappedInjection<X>(pit
				.getInjectionTarget(), entry));
	}
}
