package dto.extension;

import java.lang.reflect.Field;
import java.util.Set;

import javax.enterprise.context.spi.CreationalContext;
import javax.enterprise.inject.InjectionException;
import javax.enterprise.inject.spi.*;

import dto.*;
import org.w3c.dom.Element;

public class XmlWrappedInjection<X extends BaseDto> implements
		InjectionTarget<X> {
	private final InjectionTarget<X> wrapped;
	private final Element xmlBacking;

	public XmlWrappedInjection(InjectionTarget<X> it, Element xmlElement) {
		wrapped = it;
		xmlBacking = xmlElement;
	}

	@Override
	public void inject(X instance, CreationalContext<X> ctx) {
		wrapped.inject(instance, ctx);

		final Class<? extends BaseDto> klass = instance.getClass();
		//yjm注：出于演示目的，这里仅反射了本类中声明的field，所以注入时，父类中的field会被忽略，大家可以自行修改，逐层向上反射，直到BaseDto类为止
		for (Field field : klass.getDeclaredFields()) {
			field.setAccessible(true);
			final String fieldValueFromXml = xmlBacking.getAttribute(field
					.getName());
			try {
				//System.out.println("the filed name is :" + field.getName());
				if (field.getName().toLowerCase().equals("serialversionuid")) {
					continue;
				}
				//注：出于演示目的，这里只处理了int、long、String这三种类型，其它类型大家可自行扩展
				if (field.getType().isAssignableFrom(Integer.TYPE)) {
					field.set(instance, Integer.parseInt(fieldValueFromXml));
				} else if (field.getType().isAssignableFrom(Long.TYPE)) {
					field.set(instance, Long.parseLong(fieldValueFromXml));
				} else if (field.getType().isAssignableFrom(String.class)) {
					field.set(instance, fieldValueFromXml);
				} else {
					throw new InjectionException("Cannot convert to type "
							+ field.getType());
				}
			} catch (IllegalAccessException e) {
				throw new InjectionException("Cannot access field " + field);
			}
		}
	}

	@Override
	public void postConstruct(X instance) {
		wrapped.postConstruct(instance);
	}

	@Override
	public void preDestroy(X instance) {
		wrapped.preDestroy(instance);
	}

	@Override
	public X produce(CreationalContext<X> ctx) {
		return wrapped.produce(ctx);
	}

	@Override
	public void dispose(X instance) {
		wrapped.dispose(instance);
	}

	@Override
	public Set<InjectionPoint> getInjectionPoints() {
		return wrapped.getInjectionPoints();
	}
}
